@@ -1,6 +1,6 @@
 ##' Hitching C snippets and R functions to pomp_fun objects
 ##'
-##' The algorithms in \pkg{pomp} are formulated using \R functions that access the \link[=basic components]{basic model components}
+##' The algorithms in \pkg{pomp} are formulated using \R functions that access the \link[=basic_components]{basic model components}
 ##' (\code{rprocess}, \code{dprocess}, \code{rmeasure}, \code{dmeasure}, etc.).
 ##' For short, we refer to these elementary functions as \dQuote{\link{workhorses}}.
 ##' In implementing a model, the user specifies basic model components
@@ -8,60 +8,50 @@
 ##' Each component is then packaged into a \sQuote{pomp_fun} objects, which gives a uniform interface.
 ##' The construction of \sQuote{pomp_fun} objects is handled by the \code{hitch} function,
 ##' which conceptually \dQuote{hitches} the workhorses to the user-defined procedures.
-##'
 ##' @name hitch
 ##' @docType methods
-##' @include pomp_class.R csnippet.R safecall.R templates.R
-##'
+##' @include pomp_class.R csnippet.R safecall.R templates.R pomp_fun.R
 ##' @importFrom digest digest
 ##' @importFrom stats runif
-##'
 ##' @param \dots named arguments representing the user procedures to be hitched.
 ##' These can be functions, character strings naming routines in external, dynamically-linked libraries, C snippets, or \code{NULL}.
 ##' The first three are converted by \code{hitch} to \sQuote{pomp_fun} objects which perform the indicated computations.
 ##' \code{NULL} arguments are translated to default \sQuote{pomp_fun} objects.
 ##' If any of these procedures are already \sQuote{pomp_fun} objects, they are returned unchanged.
-##'
 ##' @param templates named list of templates.
 ##' Each workhorse must have a corresponding template.
 ##' See \code{pomp:::workhorse_templates} for a list.
-##'
 ##' @param obsnames,statenames,paramnames,covarnames character vectors
 ##' specifying the names of observable variables, latent state variables, parameters, and covariates, respectively.
 ##' These are only needed if one or more of the horses are furnished as C snippets.
-##'
 ##' @param PACKAGE optional character;
 ##' the name (without extension) of the external, dynamically loaded library in which any native routines are to be found.
 ##' This is only useful if one or more of the model components has been specified using a precompiled dynamically loaded library;
 ##' it is not used for any component specified using C snippets.
 ##' \code{PACKAGE} can name at most one library.
-##'
-##' @param globals optional character;
+##' @param globals optional character or C snippet;
 ##' arbitrary C code that will be hard-coded into the shared-object library created when C snippets are provided.
 ##' If no C snippets are used, \code{globals} has no effect.
-##'
+##' @param on_load optional character or C snippet:
+##' arbitrary C code that will be executed when the C snippet file is loaded.
+##' If no C snippets are used, \code{on_load} has no effect.
 ##' @param cdir optional character variable.
 ##' \code{cdir} specifies the name of the directory within which C snippet code will be compiled.
 ##' By default, this is in a temporary directory specific to the \R session.
 ##' One can also set this directory using the \code{pomp_cdir} global option.
-##'
 ##' @param cfile optional character variable.
 ##' \code{cfile} gives the name of the file (in directory \code{cdir}) into which C snippet codes will be written.
 ##' By default, a random filename is used.
 ##' If the chosen filename would result in over-writing an existing file, an error is generated.
-##'
 ##' @param shlib.args optional character variables.
 ##' Command-line arguments to the \code{R CMD SHLIB} call that compiles the C snippets.
 ##' One can, for example, specify libraries against which the C snippets are to be linked.
 ##' In doing so, take care to make sure the appropriate header files are available to the C snippets, e.g., using the \code{globals} argument.
 ##' See \code{\link{Csnippet}} for more information.
-##'
 ##' @param compile logical;
 ##' if \code{FALSE}, compilation of the C snippets will be postponed until they are needed.
-##'
 ##' @param verbose logical.
 ##' Setting \code{verbose=TRUE} will cause additional information to be displayed.
-##'
 ##' @return
 ##' \code{hitch} returns a named list of length two.  The element named
 ##' \dQuote{funs} is itself a named list of \sQuote{pomp_fun} objects, each of
@@ -77,9 +67,7 @@
 ##' library was created in the session's temporary directory.  }
 ##' \item{src}{ A
 ##' character string with the full contents of the C snippet file.  } }
-##'
 ##' @author Aaron A. King
-##'
 ##' @seealso \code{\link{pomp}}, \code{\link{spy}}
 ##' @concept extending the pomp package
 ##' @concept low-level interface
@@ -95,7 +83,7 @@ NULL
 hitch <- function (..., templates,
   obsnames, statenames, paramnames, covarnames,
   PACKAGE, globals, cfile, cdir = getOption("pomp_cdir", NULL),
-  shlib.args, compile = TRUE,
+  on_load, shlib.args, compile = TRUE,
   verbose = getOption("verbose", FALSE)) {
 
   if (missing(templates))
@@ -105,6 +93,7 @@ hitch <- function (..., templates,
   if (missing(cdir)) cdir <- NULL
   if (missing(PACKAGE)) PACKAGE <- NULL
   if (missing(globals)) globals <- NULL
+  if (missing(on_load)) on_load <- NULL
   if (missing(shlib.args)) shlib.args <- NULL
   PACKAGE <- as.character(PACKAGE)
   compile <- as.logical(compile)
@@ -144,6 +133,7 @@ hitch <- function (..., templates,
             paramnames=paramnames,
             covarnames=covarnames,
             globals=globals,
+            on_load=on_load,
             shlib.args=shlib.args,
             compile=compile,
             verbose=verbose
@@ -183,7 +173,7 @@ hitch <- function (..., templates,
 
 Cbuilder <- function (..., templates, name = NULL, dir = NULL,
   statenames, paramnames, covarnames, obsnames,
-  globals, shlib.args = NULL, compile,
+  globals, on_load, shlib.args = NULL, compile,
   verbose = getOption("verbose",FALSE))
 {
 
@@ -194,6 +184,7 @@ Cbuilder <- function (..., templates, name = NULL, dir = NULL,
   obsnames <- cleanForC(obsnames)
 
   globals <- as(globals,"character")
+  on_load <- as(on_load,"character")
 
   snippets <- list(...)
 
@@ -203,8 +194,12 @@ Cbuilder <- function (..., templates, name = NULL, dir = NULL,
   utils <- which(
     vapply(
       seq_along(pomp_templates$utilities),
-      \(x) any(grepl(pomp_templates$utilities[[x]]$trigger,
-        snippets)),
+      \(x) any(
+             grepl(
+               pomp_templates$utilities[[x]]$trigger,
+               c(snippets,on_load)
+             )
+           ),
       logical(1L)
     )
   )
@@ -226,7 +221,7 @@ Cbuilder <- function (..., templates, name = NULL, dir = NULL,
   out <- textConnection(object="csrc",open="w",local=TRUE)
 
   cat(file=out,render(pomp_templates$file$header,
-    pompheader=pompheader,timestamp=timestamp,salt=salt))
+    pompheader=pompheader,timestamp=timestamp,salt=salt),"\n")
 
   cat(file=out,globals,"\n\n")
 
@@ -238,21 +233,23 @@ Cbuilder <- function (..., templates, name = NULL, dir = NULL,
     ## we add each 'Cname' to the 'registry'
     registry <- c(registry,templates[[snip]]$Cname)
     ## define variables
-    cat(file=out,render("\n/* C snippet: '{%snip%}' */\n",snip=snip))
+    cat(file=out,render("\n\n/* C snippet: '{%snip%}' */\n",snip=snip))
     for (k in seq_along(templates[[snip]]$vars)) {
       vtpl <- templates[[snip]]$vars[[k]]
       nm <- eval(vtpl$names)
       for (v in seq_along(nm)) {
-        cat(file=out,render(
-          pomp_templates$define,
-          variable=nm[v],
-          cref=render(vtpl$cref,v=v-1)
-        ))
+        cat(
+          file=out,
+          render(
+            pomp_templates$define,
+            variable=nm[v],
+            cref=render(vtpl$cref,v=v-1)
+          ))
       }
     }
     ## render the C snippet in context
-    cat(file=out,render(templates[[snip]]$header),
-      snippets[[snip]],templates[[snip]]$footer)
+    cat(file=out,"\n",render(templates[[snip]]$header),
+      snippets[[snip]],templates[[snip]]$footer,"\n")
     ## undefine variables
     for (k in seq_along(templates[[snip]]$vars)) {
       vtpl <- templates[[snip]]$vars[[k]]
@@ -264,15 +261,16 @@ Cbuilder <- function (..., templates, name = NULL, dir = NULL,
   }
 
   ## load/unload stack handling codes
-  cat(file=out,pomp_templates$stackhandling)
+  cat(file=out,pomp_templates$stackhandling,"\n")
 
   ## registration
-  cat(file=out,render(pomp_templates$registration$header))
+  cat(file=out,"\n",render(pomp_templates$registration$header))
   for (v in utils)
     cat(file=out,pomp_templates$utilities[[v]]$reg)
   for (v in registry)
     cat(file=out,render(pomp_templates$registration$main,fun=v))
-  cat(file=out,pomp_templates$registration$footer)
+  cat(file=out,"\n",on_load,"\n") # code to be executed at library load
+  cat(file=out,pomp_templates$registration$footer,"\n")
 
   close(out)
 
@@ -332,17 +330,17 @@ pompCompile <- function (fname, direc, src, shlib.args = NULL,
     if (verbose) cat("compiling",sQuote(solib),"\n")
 
     tryCatch(
-      {
-        rv <- system2(
-          command=R.home("bin/R"),
-          args=c("CMD","SHLIB","-c","-o",solib,modelfile,shlib.args),
-          env=cflags,
-          wait=TRUE,
-          stdout=TRUE,
-          stderr=TRUE
-        )
-      },
-      error = function (e) pStop_("error compiling C snippets: ",conditionMessage(e)) #nocov
+    {
+      rv <- system2(
+        command=R.home("bin/R"),
+        args=c("CMD","SHLIB","-c","-o",solib,modelfile,shlib.args),
+        env=cflags,
+        wait=TRUE,
+        stdout=TRUE,
+        stderr=TRUE
+      )
+    },
+    error = function (e) pStop_("error compiling C snippets: ",conditionMessage(e)) #nocov
     )
 
     stat <- as.integer(attr(rv,"status"))
@@ -366,11 +364,11 @@ srcDir <- function (dir, verbose) {
   if (!dir.exists(dir)) {
     if (verbose) cat("creating C snippet directory ",sQuote(dir),"\n") #nocov
     tryCatch(
-      {
-        dir.create(dir,recursive=TRUE,showWarnings=FALSE,mode="0700")
-        stopifnot(dir.exists(dir))
-      },
-      error = function (e) pStop_("cannot create cache directory ",sQuote(dir))   #nocov
+    {
+      dir.create(dir,recursive=TRUE,showWarnings=FALSE,mode="0700")
+      stopifnot(dir.exists(dir))
+    },
+    error = function (e) pStop_("cannot create cache directory ",sQuote(dir))   #nocov
     )
   }
   dir
@@ -405,42 +403,3 @@ render <- function (template, ...) {
   }
   do.call(paste0,retval)
 }
-
-## TEMPLATES
-
-pomp_templates <- list(
-  define="#define {%variable%}\t\t({%cref%})\n",
-  undefine="#undef {%variable%}\n",
-  file=list(
-    header="/* pomp C snippet file: {%name%} */\n/* Time: {%timestamp%} */\n/* Salt: {%salt%} */\n\n#include <{%pompheader%}>\n#include <R_ext/Rdynload.h>\n\n"
-  ),
-  utilities=list(
-    periodic_bspline_basis=list(
-      trigger="periodic_bspline_basis_eval",
-      header="static periodic_bspline_basis_eval_t *__pomp_periodic_bspline_basis_eval;\n#define periodic_bspline_basis_eval(X,Y,M,N,Z)\t(__pomp_periodic_bspline_basis_eval((X),(Y),(M),(N),(Z)))
-\n",
-      reg="__pomp_periodic_bspline_basis_eval = (periodic_bspline_basis_eval_t *) R_GetCCallable(\"pomp\",\"periodic_bspline_basis_eval\");\n"
-    ),
-    get_userdata_int=list(
-      trigger="get_userdata_int",
-      header="static get_userdata_int_t *__pomp_get_userdata_int;\n#define get_userdata_int(X)\t(__pomp_get_userdata_int(X))\n",
-      reg="__pomp_get_userdata_int = (get_userdata_t *) R_GetCCallable(\"pomp\",\"get_userdata_int\");\n"
-    ),
-    get_userdata_double=list(
-      trigger="get_userdata_double",
-      header="static get_userdata_double_t *__pomp_get_userdata_double;\n#define get_userdata_double(X)\t(__pomp_get_userdata_double(X))\n",
-      reg="__pomp_get_userdata_double = (get_userdata_double_t *) R_GetCCallable(\"pomp\",\"get_userdata_double\");\n"
-    ),
-    get_userdata=list(
-      trigger="get_userdata(\\b|[^_])",
-      header="static get_userdata_t *__pomp_get_userdata;\n#define get_userdata(X)\t(__pomp_get_userdata(X))\n",
-      reg="__pomp_get_userdata = (get_userdata_t *) R_GetCCallable(\"pomp\",\"get_userdata\");\n"
-    )
-  ),
-  stackhandling="\nstatic int __pomp_load_stack = 0;\n\nvoid __pomp_load_stack_incr (void) {++__pomp_load_stack;}\n\nvoid __pomp_load_stack_decr (int *val) {*val = --__pomp_load_stack;}\n",
-  registration=list(
-    header="\nvoid R_init_{%name%} (DllInfo *info)\n{\n",
-    main="R_RegisterCCallable(\"{%name%}\", \"{%fun%}\", (DL_FUNC) {%fun%});\n",
-    footer="}\n\n"
-  )
-)
