@@ -7,11 +7,9 @@ static const R_CallMethodDef callMethods[] = {
   {"bspline_basis", (DL_FUNC) &bspline_basis, 5},
   {"periodic_bspline_basis", (DL_FUNC) &periodic_bspline_basis, 5},
   {"systematic_resampling", (DL_FUNC) &systematic_resampling, 2},
-  {"euler_model_simulator", (DL_FUNC) &euler_model_simulator, 11},
   {"lookup_in_table", (DL_FUNC) &lookup_in_table, 2},
   {"load_stack_incr", (DL_FUNC) &load_stack_incr, 1},
   {"load_stack_decr", (DL_FUNC) &load_stack_decr, 1},
-  {"SSA_simulator", (DL_FUNC) &SSA_simulator, 11},
   {"R_Euler_Multinom", (DL_FUNC) &R_Euler_Multinom, 4},
   {"D_Euler_Multinom", (DL_FUNC) &D_Euler_Multinom, 5},
   {"R_GammaWN", (DL_FUNC) &R_GammaWN, 3},
