@@ -5,16 +5,9 @@
 static SEXP __pomp_ptr_userdata;
 #define USERDATA  (__pomp_ptr_userdata)
 
-void set_pomp_userdata (SEXP userdata) {
-#if (R_VERSION < 262659) // before 4.2.3
-  if (LENGTH(userdata) > 0) {
-    USERDATA = userdata;
-  } else {
-    USERDATA = R_NilValue;
-  }
-#else
+SEXP set_pomp_userdata (SEXP userdata) {
   USERDATA = userdata;
-#endif
+  return R_NilValue;
 }
 
 const SEXP get_userdata (const char *name) {
